#include <stdio.h>
#include <gtk/gtk.h>

static gboolean button_press(GtkWidget *widget, GdkEvent *event)
{
  GdkEventButton *button_event;

  if(event->type == GDK_BUTTON_PRESS) {
    button_event = (GdkEventButton *)event;
    gtk_menu_popup(GTK_MENU(widget), NULL, NULL, NULL, NULL, 
        button_event->button, button_event->time);
    return TRUE;
  }

  return FALSE;
}

static GtkWidget *create_message_item(const char *summary, const char *body,
    const char *timestamp, const char *app_name, const char *icon_name)
{
  GtkWidget *menu_item;
  GtkWidget *hbox;
  GtkWidget *image;
  GtkWidget *label;
  char buffer[1024];

  menu_item = gtk_menu_item_new();

  sprintf(buffer, "<b>%s</b>\n%s\n<small><i>%s from <b>%s</b></i></small>", summary,
      body, timestamp, app_name);

  image = gtk_image_new_from_icon_name(icon_name, GTK_ICON_SIZE_DIALOG);
  gtk_widget_show(image);

  label = gtk_label_new(NULL);
  gtk_label_set_markup(GTK_LABEL(label), buffer);
  gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
  gtk_widget_show(label);

  hbox = gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox), image, FALSE, FALSE, 15);
  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
  gtk_container_add(GTK_CONTAINER(menu_item), hbox);
  gtk_widget_show(hbox);

  gtk_widget_show(menu_item);

  return menu_item;
}

int main(int argc, char *argv[])
{
  GtkWidget *window;
  GtkWidget *menu;
  GtkWidget *menu_items;
  GtkWidget *vbox;
  GtkWidget *button;

  gtk_init(&argc, &argv);

  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_widget_set_size_request(window, 200, 100);
  gtk_window_set_title(GTK_WINDOW(window), "Menu Example");
  gtk_container_set_border_width(GTK_CONTAINER(window), 10);

  g_signal_connect(G_OBJECT(window), "delete_event", G_CALLBACK(gtk_main_quit), NULL);

  menu = gtk_menu_new();
  
  menu_items = create_message_item("Your Friend", "I just wanted to say hello.",
      "2 minutes ago", "Pidgin", "pidgin");
  gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_items);

  menu_items = create_message_item("Some Guy", "I had cake for lunch #cakelunchie",
      "1 hour ago", "Gwibber", "gwibber");
  gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_items);

  vbox = gtk_vbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER(window), vbox);
  gtk_widget_show(vbox);

  button = gtk_button_new_with_label("Example");
  g_signal_connect_swapped(G_OBJECT(button), "event", G_CALLBACK(button_press), G_OBJECT(menu));
  gtk_box_pack_start(GTK_BOX(vbox), button, TRUE, TRUE, 0);
  gtk_widget_show(button);

  gtk_widget_show(window);

  gtk_main();

  return 0;
}
